; *******************************************************************
; *** This software is copyright 2004 by Michael H Riley          ***
; *** You have permission to use, modify, copy, and distribute    ***
; *** this software so long as this copyright notice is retained. ***
; *** This software may not be used in commercial applications    ***
; *** without express written permission from the author.         ***
; *******************************************************************

include    bios.inc
include    kernel.inc

           org     8000h
           db      'COPY',0
           dw      9000h
           dw      endrom+7000h
           dw      2000h
           dw      endrom-2000h
           dw      2000h
           db      0

           org     2000h
           br      start

include    date.inc

start:     ghi     ra                  ; copy argument address to rf
           phi     rf
           glo     ra
           plo     rf
loop1:     lda     rf                  ; look for first less <= space
           smi     33
           bdf     loop1
           ghi     rf                  ; make copy of 2nd filename start
           phi     r9
           glo     rf
           plo     r9
           dec     rf                  ; backup to char
           ldi     0                   ; need proper termination
           str     rf
           inc     rf                  ; now find end of 2nd filename
loop2:     lda     rf                  ; look for first less <= space
           smi     33
           bdf     loop2
           dec     rf                  ; backup to char
           ldi     0                   ; need proper termination
           str     rf

           ghi     ra                  ; back to beginning of name
           phi     rf
           glo     ra
           plo     rf
           ldi     high fildes         ; get file descriptor
           phi     rd
           ldi     low fildes
           plo     rd
           ldi     0                   ; flags for open
           plo     r7
           sep     scall               ; attempt to open file
           dw      o_open
           bnf     opened              ; jump if file was opened
           ldi     high errmsg         ; get error message
           phi     rf
           ldi     low errmsg
           plo     rf
           sep     scall               ; display it
           dw      f_msg
           lbr     o_wrmboot           ; and return to os
opened:    ghi     rd                  ; make copy of descriptor
           phi     r7
           glo     rd
           plo     r7
           ghi     r9                  ; back to beginning of name
           phi     rf
           glo     r9
           plo     rf
           ldi     high dfildes        ; get file descriptor
           phi     rd
           ldi     low dfildes
           plo     rd
           glo     r7                  ; save first descriptor
           stxd
           ghi     r7
           stxd
           ldi     3                   ; flags for open, create if nonexist
           plo     r7
           sep     scall               ; attempt to open file
           dw      o_open
           irx                         ; recover first descriptor
           ldxa
           phi     r7
           ldx
           plo     r7
           ghi     rd                  ; make copy of descriptor
           phi     r8
           glo     rd
           plo     r8
mainlp:    ldi     0                   ; want to read 255 bytes
           phi     rc
           ldi     255
           plo     rc 
           ldi     high buffer         ; buffer to rettrieve data
           phi     rf
           ldi     low buffer
           plo     rf
           ghi     r7                  ; get descriptor
           phi     rd
           glo     r7
           plo     rd
           sep     scall               ; read the header
           dw      o_read
           glo     rc                  ; check for zero bytes read
           bz      done                ; jump if so
           ldi     high buffer         ; buffer to rettrieve data
           phi     rf
           ldi     low buffer
           plo     rf
           ghi     r8                  ; get descriptor
           phi     rd
           glo     r8
           plo     rd
           sep     scall               ; write to destination file
           dw      o_write
           br      mainlp              ; loop back til done


done:      sep     scall               ; close the file
           dw      o_close
           ghi     r8                  ; get destination descriptor
           phi     rd
           glo     r8
           plo     rd
           sep     scall               ; and close it
           dw      o_close
           lbr     o_wrmboot
           sep     sret                ; return to os



           

errmsg:    db      'File not found',10,13,0
fildes:    db      0,0,0,0
           dw      dta
           db      0,0
           db      0
           db      0,0,0,0
           dw      0,0
           db      0,0,0,0
dfildes:   db      0,0,0,0
           dw      ddta
           db      0,0
           db      0
           db      0,0,0,0
           dw      0,0
           db      0,0,0,0

endrom:    equ     $

dta:       ds      512
ddta:      ds      512
buffer:    db      0

